/* main.vala
 *
 * Copyright 2024 NekoCWD
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */
Alsa.ElemId speaker_id;
Alsa.ElemId earpiece_id;

string speaker_mixer;
string earpiece_mixer;
string card_name;

private const GLib.OptionEntry[] options = {
    { "speaker", 's', OptionFlags.IN_MAIN, OptionArg.STRING, ref speaker_mixer, "mixer for speaker mode", "QUAT_MI2S_RX Voice Mixer VoiceMMode1" },
    { "earpiece", 'e', OptionFlags.IN_MAIN, OptionArg.STRING, ref earpiece_mixer, "mixer for earpiece mode", "SLIMBUS_0_RX Voice Mixer VoiceMMode1" },
    { "card", 'c', OptionFlags.IN_MAIN, OptionArg.STRING, ref card_name, "alsa card", "hw:0" },
    { null }
};

void get_mixes (string earpiece, string speaker) {
    var list = new List<Alsa.ElemId> ();
    Alsa.Card card;
    Alsa.ElemList elem_list;
    Alsa.ElemId elem_id;
    Alsa.Card.open (out card, card_name);
    Alsa.ElemList.alloc (out elem_list);
    Alsa.ElemId.alloc (out elem_id);
    card.elem_list (elem_list); // Getting list without items
    elem_list.alloc_space (elem_list.get_count ()); // Allocating items
    card.elem_list (elem_list); // Getting list with items
    for (uint i = 0; i < elem_list.get_count (); i++) {
        elem_list.get_id (i, elem_id);

        var elem_name = elem_id.get_name ();
        if (elem_name == speaker) {
            Alsa.ElemId.alloc (out speaker_id);
            elem_list.get_id (i, speaker_id);
        } else if (elem_name == earpiece) {
            Alsa.ElemId.alloc (out earpiece_id);
            elem_list.get_id (i, earpiece_id);
        }
        if (elem_name.contains ("Voice Mixer VoiceMMode1")) {
            message ("Available %s", elem_id.get_name ());
        }
    }
    if (speaker_id != null) {
        message ("Speaker Setted %s", speaker_id.get_name ());
    } else {
        error ("Speaker mixer not set");
    }
    if (earpiece_id != null) {
        message ("Earpiece Setted %s", earpiece_id.get_name ());
    } else {
        error ("Earpiece mixer not set");
    }
}

int main (string[] args) {

    try {
        var opt_context = new OptionContext ("\n\tThis program will receive the callaudiod signal then call q6meowd to stop the stream, then set the ALSA ctl to speaker or earpeace, then call q6meowd to start the stream again");
        opt_context.set_help_enabled (true);
        opt_context.add_main_entries (options, null);
        opt_context.parse (ref args);
    } catch (OptionError e) {
        printerr ("error: %s\n", e.message);
        printerr ("Run '%s --help' to see a full list of available command line options.\n", args[0]);
        return 1;
    }
    if (speaker_mixer == null || earpiece_mixer == null || card_name == null) {
        error ("Run --help to get options. You need to specify card, speaker and earpiece");
    }

    Alsa.Card card;
    message ("Card open: %d", Alsa.Card.open (out card, card_name, Alsa.CardOpenType.NONBLOCK));

    get_mixes (earpiece_mixer, speaker_mixer);
    Q6MeowD q6meowd_proxy = Bus.get_proxy_sync (BusType.SYSTEM, "org.postmarketos.q6meowd", "/org/postmarketos/q6meowd", DBusProxyFlags.NONE);
    CallAudioD callaudiod_proxy = Bus.get_proxy_sync (BusType.SESSION, "org.mobian_project.CallAudio", "/org/mobian_project/CallAudio", DBusProxyFlags.NONE);
    ((DBusProxy) callaudiod_proxy).g_properties_changed.connect ((props, inv) => {
        foreach (var prop in props) {
            var name = prop.get_child_value (0).get_string ();
            if (name == "SpeakerState") {
                var is_speaker = prop.get_child_value (1).get_variant ().get_uint32 () == 1;
                q6meowd_proxy.stop_stream ();
                Alsa.ElemValue ctl;
                Alsa.ElemValue.alloc (out ctl);
                if (is_speaker) {
                    ctl.set_id (speaker_id);
                    message ("Enabling speaker %s", speaker_id.get_name ());
                } else {
                    ctl.set_id (earpiece_id);
                    message ("Enabling earpiece %s", earpiece_id.get_name ());
                }
                ctl.set_boolean (1, true);
                // card.elem_write (ctl);
                q6meowd_proxy.start_stream ();
            }
        }
    });
    new MainLoop ().run ();
    return 0;
}